 body {
    overflow-x: hidden;
 }


 .topbar {
    background: #1034A6;
    color : #FFFFFF;
 }

.ham {
    color : #FFFFFF;
 }
 .ham:hover {
    background: #FFFFFF;
    color : #000000;
 }

 .ham:active,
 .ham:focus {
    background: #FFFFFF;
    color : #000000;
 }

.page-header {
    border-color: #000;
}

.desc-panel {
    border-radius: 0px;
    border : 0px;
    border-left: 1px solid;
    border-color: #041451;
    box-shadow: none;
}

.panel-group .panel+.panel {
    margin-top: 20px;
}

.desc-panel > .panel-heading {
    background: #e5e6e8;
    border-radius: 0px;
}

.desc-panel-heading {
    padding : 0px;
    overflow : auto;
    height : auto;
    border-radius: 0px;
    background: #e5e6e8;
}

.desc-panel-body {
    background: #f9f9f9;
}

.desc-panel-footer {
    padding-left : 10px;
    padding-right: 0px;
    padding-bottom: 1px;
}

.desc-panel-group {
    margin : 0px;
}

.desc-type {
    font-size : 16px;
    color : #FFF;
    font-weight: bold;
    background: #041051;
    width : 10%;
    float : left;
    padding : 6px;
}

.desc-sign {
    font-size: 16px;
    background: #e5e6e8;
    color : #000;
    width : 70%;
    float : left;
    padding: 6px;
}

.desc-btns {
    float : left;
    height : 100%;
}

.inherittype {
    background : #C40233;
    color : #FFF;
    padding : 4px;
    border-radius: 3px;
    margin-left: 6px;
    margin-right: 6px;
}

.doc-type {
    font-size : 16px;
    color : #111;
    font-weight: bold;
    background: #F4A966;
    height : 100%;
    width : 10%;
    float : left;
    padding : 6px;
}

.sign-list {
    font-size: 16px;
     background: #eaecef;
}

.ret {
    width : 20%;
    color : #d80d0d;
}

.name {
    width : 10%;
    font-weight: bold;
    color : #000000;
}

.param {
    width : 70%;
    color : #550982;
}


/* Toggle Styles */

#wrapper {
    padding-left: 0;
    -webkit-transition: all 0.5s ease;
    -moz-transition: all 0.5s ease;
    -o-transition: all 0.5s ease;
    transition: all 0.5s ease;
}

#wrapper.toggled {
    padding-left: 250px;
}

#sidebar-wrapper {
    z-index: 1000;
    position: fixed;
    left: 250px;
    width: 0;
    height: 100%;
    margin-left: -250px;
    overflow-y: auto;
    background: #004953;
    -webkit-transition: all 0.5s ease;
    -moz-transition: all 0.5s ease;
    -o-transition: all 0.5s ease;
    transition: all 0.5s ease;
}

#wrapper.toggled #sidebar-wrapper {
    width: 250px;
}

#page-content-wrapper {
    width: 100%;
    position: absolute;
}

#wrapper.toggled #page-content-wrapper {
    position: absolute;
    margin-right: -250px;
}

/* Sidebar Styles */

.sidebar-nav {
    position: absolute;
    top: 0;
    width: 250px;
    margin: 0;
    padding: 0;
    list-style: none;
}

.sidebar-nav li {
    text-indent: 20px;
    line-height: 40px;
}

.sidebar-nav li a {
    display: block;
    text-decoration: none;
    color: #fff;
}

.sidebar-nav li a:hover {
    text-decoration: none;
    color: #fff;
    background: rgba(255,255,255,0.5);
}

.sidebar-nav li a:active,
.sidebar-nav li a:focus {
    text-decoration: none;
}

.sidebar-nav > .sidebar-brand {
    height: 65px;
    font-size: 18px;
    line-height: 60px;
}

.sidebar-nav > .sidebar-brand a {
    color: #dddddd;
}

.sidebar-nav > .sidebar-brand a:hover {
    color: #fff;
    background: none;
}


@media(min-width:768px) {
    #wrapper {
        padding-left: 0;
    }

    #wrapper.toggled {
        padding-left: 250px;
    }

    #sidebar-wrapper {
        width: 0;
    }

    #wrapper.toggled #sidebar-wrapper {
        width: 250px;
    }

    #page-content-wrapper {
        padding: 10px;
        position: relative;
    }

    #wrapper.toggled #page-content-wrapper {
        position: relative;
        margin-right: 0;
    }
}
